package com.cbr.hdt.impl.spark;

import com.cbr.hdt.spark.Stats;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URI;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.Map;
import java.util.SortedMap;
import java.util.TreeMap;
import java.util.concurrent.atomic.AtomicInteger;
import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.fs.FileSystem;
import org.apache.hadoop.fs.LocatedFileStatus;
import org.apache.hadoop.fs.Path;
import org.apache.hadoop.fs.RemoteIterator;
import org.apache.parquet.column.page.PageReadStore;
import org.apache.parquet.example.data.Group;
import org.apache.parquet.example.data.simple.convert.GroupRecordConverter;
import org.apache.parquet.hadoop.ParquetFileReader;
import org.apache.parquet.hadoop.metadata.ParquetMetadata;
import org.apache.parquet.io.ColumnIOFactory;
import org.apache.parquet.io.MessageColumnIO;
import org.apache.parquet.io.RecordReader;
import org.apache.parquet.schema.MessageType;
import org.apache.spark.sql.Dataset;
import org.apache.spark.sql.Row;
import static org.apache.spark.sql.functions.count;
import static org.apache.spark.sql.functions.sum;
import static org.apache.spark.sql.functions.length;
import static org.apache.spark.sql.functions.max;
import static org.apache.spark.sql.functions.min;
import org.rdfhdt.hdt.compact.integer.VByte;
import org.rdfhdt.hdt.compact.sequence.SequenceLog64Big;
import org.rdfhdt.hdt.dictionary.DictionarySectionPrivate;
import org.rdfhdt.hdt.dictionary.TempDictionarySection;
import static org.rdfhdt.hdt.dictionary.impl.section.PFCDictionarySectionBig.BLOCK_PER_BUFFER;
import org.rdfhdt.hdt.listener.ProgressListener;
import org.rdfhdt.hdt.options.HDTOptions;
import org.rdfhdt.hdt.util.crc.CRC32;
import org.rdfhdt.hdt.util.crc.CRC8;
import org.rdfhdt.hdt.util.crc.CRCOutputStream;
import org.rdfhdt.hdt.util.io.IOUtil;
import org.rdfhdt.hdt.util.string.ByteStringUtil;

/**
 *
 * @author carlos
 */
public class SparkDictSection implements DictionarySectionPrivate {

    /**
     *
     */
    public static final int TYPE_INDEX = 2;

    /**
     *
     */
    public static final int DEFAULT_BLOCK_SIZE = 16;

    /**
     *
     */
    protected int blocksize;

    /**
     *
     */
    protected int numstrings;

    /**
     *
     */
    protected long size;
    static AtomicInteger filecounter = new AtomicInteger(0);

    SequenceLog64Big blocks;

    ByteArrayOutputStream byteOut;
    CharSequence previousStr;
    int buffer;
    int blockPerBuffer;
    long storedBuffersSize;

    ArrayList<ArrayList<byte[]>> data; // to exceed 2GB block size
    long[] posFirst;

    long firstKey;
    long lastKey;

    Dataset<Row> df;

    Configuration conf = null;
    FileSystem fileSystem = null;
    Path path = null;

    String sectionName;

    /**
     *
     * @param spec
     * @param sectionName
     * @param df
     */
    public SparkDictSection(HDTOptions spec, String sectionName, Configuration conf, String pathString, Dataset<Row> df) throws Exception {
        this.blocksize = (int) spec.getInt("pfc.blocksize");
        this.sectionName = sectionName;
        if (blocksize == 0) {
            blocksize = DEFAULT_BLOCK_SIZE;
        }
        this.path = new Path(pathString);
        this.conf = conf;
        this.conf.addResource("/etc/hadoop/conf/core-site.xml");
        this.conf.addResource("/etc/hadoop/conf/hdfs-site.xml");
        this.conf.set("fs.default.name", "hdfs://hadoop");
        this.fileSystem = FileSystem.get(new URI("hdfs://hadoop"), this.conf);
        this.df = df;
    }

    /**
     *
     * @param listener
     * @param stats
     * @throws java.io.IOException
     */
    public void loadFromSpark(ProgressListener listener, Map<String, Stats> stats) throws IOException {

        String kStats = "";
        String kStatsKey = "";
        switch (sectionName) {
            case "Subject":
                kStats = "subjStats";
                kStatsKey = "subjKeyStats";
                break;
            case "Predicate":
                kStats = "predStats";
                kStatsKey = "predKeyStats";
                break;
            case "Object":
                kStats = "objStats";
                kStatsKey = "objKeyStats";
                break;
            case "Shared":
                kStats = "soStats";
                kStatsKey = "soKeyStats";
                break;
        }

        if (stats.containsKey(kStats) && stats.containsKey(kStatsKey)) {
            size = stats.get(kStatsKey).getCount() + 1;
            numstrings = (int) stats.get(kStats).getCount();
            firstKey = stats.get(kStats).getMin();
            lastKey = stats.get(kStats).getMax();
        } else {
            System.out.println(String.format("\t\tComputing %s section stats...", sectionName));
            Row rstats = df.agg(
                    sum(length(df.col("key")).plus(1L)),
                    count(df.col("key")),
                    min(df.col("value")),
                    max(df.col("value"))
            ).first();

            size = rstats.getLong(0);
            numstrings = (int) rstats.getLong(1);
            firstKey = rstats.getLong(2);
            lastKey = rstats.getLong(3);
        }

        System.out.println(String.format("\t\tProcessing %s section: %d strings.",
                sectionName,
                numstrings
        ));

        this.blocks = new SequenceLog64Big(64, numstrings / blocksize);
        int index = 0;

        String name = ".test" + filecounter.getAndIncrement() + ".tmp";
        File file = new File(name);
        FileOutputStream out = null;

        try {
            if (!file.exists()) {
                file.createNewFile();
            }
            out = new FileOutputStream(file);
        } catch (FileNotFoundException e1) {
            // TODO Auto-generated catch block
            e1.printStackTrace();
        } catch (IOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }

        long byteoutsize = 0;
        ByteArrayOutputStream byteOut = new ByteArrayOutputStream(16 * 1024);
        CharSequence previousStr = null;

        try {
            SortedMap<String, Path> sortedPaths = new TreeMap<>();

            System.out.println(String.format("\t\tReading %s", path.toUri().toString()));
            for (RemoteIterator<LocatedFileStatus> it = fileSystem.listFiles(path, false); it.hasNext();) {
                LocatedFileStatus fileStatus = it.next();
                Path filePath = fileStatus.getPath();
                String fileName = filePath.getName();
                if (!fileName.startsWith(".") && !fileName.startsWith("_") && !fileName.startsWith("tmp")) {
                    sortedPaths.put(fileName, filePath);
                }
            }
            for (Path filePath : sortedPaths.values()) {
                //System.out.println(String.format("\t\tReading %s", filePath.toUri().toString()));
                ParquetMetadata readFooter = ParquetFileReader.readFooter(conf, filePath);
                MessageType schema = readFooter.getFileMetaData().getSchema();
                ParquetFileReader parquetFileReader = new ParquetFileReader(conf, filePath, readFooter.getBlocks(), schema.getColumns());
                PageReadStore pages = null;
                try {
                    while (null != (pages = parquetFileReader.readNextRowGroup())) {
                        final long rows = pages.getRowCount();

                        final MessageColumnIO columnIO = new ColumnIOFactory().getColumnIO(schema);
                        final RecordReader<Group> recordReader = columnIO.getRecordReader(pages, new GroupRecordConverter(schema));
                        for (int i = 0; i < rows; i++) {
                            final Group row = recordReader.read();
                            CharSequence str = row.getString(0, 0);

                            if (index % blocksize == 0) {
                                // Add new block pointer
                                byteOut.flush();
                                byteoutsize = byteoutsize + byteOut.size();

                                blocks.append(byteoutsize);
                                byteOut.writeTo(out);
                                byteOut.reset();
                                // Copy full string
                                ByteStringUtil.append(byteOut, str, 0);

                            } else {
                                // Find common part.
                                int delta = ByteStringUtil.longestCommonPrefix(previousStr, str);
                                // Write Delta in VByte
                                VByte.encode(byteOut, delta);
                                // Write remaining
                                ByteStringUtil.append(byteOut, str, delta);

                            }
                            byteOut.write(0); // End of string

                            index++;

                            previousStr = str;
                        }
                    }
                } finally {
                    parquetFileReader.close();
                }
            }

            // Ending block pointer.
            byteOut.flush();
            byteoutsize = byteoutsize + byteOut.size();

            //blocks.append(byteOut.size());
            blocks.append(byteoutsize);
            // Trim text/blocks
            blocks.aggresiveTrimToSize();

            byteOut.flush();
            byteOut.writeTo(out);
            out.close();

            InputStream in = new FileInputStream(name);
            // Read block by block
            // Read packed data

            int block = 0;
            int buffer = 0;
            long bytePos = 0;
            long numBlocks = blocks.getNumberOfElements();
            //System.out.println("numblocks:"+numBlocks);

            long numBuffers = 1 + numBlocks / BLOCK_PER_BUFFER;
            int maxChunkSize = Integer.MAX_VALUE - 8;
        
            try {
                data = new ArrayList<>((int)numBuffers);
                posFirst = new long[(int) numBuffers];

                while (block < numBlocks - 1) {
                    int nextBlock = (int) Math.min(numBlocks - 1, block + BLOCK_PER_BUFFER);
                    long nextBytePos = blocks.get(nextBlock);
                    long bufferSize = nextBytePos - bytePos;

                    long pending = bufferSize;
                    
                    ArrayList<byte[]> chunks = new ArrayList<>();
                    
                    while (pending > 0) {
                        int chunkSize = (int) Math.min(maxChunkSize, pending);
                        chunks.add(IOUtil.readBuffer(in, (int) chunkSize, null));  
                        pending -= chunkSize;
                    }

                    data.add(chunks);

                    posFirst[buffer] = bytePos;

                    bytePos = nextBytePos;
                    block += BLOCK_PER_BUFFER;
                    buffer++;
                }
            } catch (Exception ex) {
                int nextBlock = (int) Math.min(numBlocks - 1, block + BLOCK_PER_BUFFER);
                long nextBytePos = blocks.get(nextBlock);

                //System.out.println("Loding block: "+i+" from "+previous+" to "+ current+" of size "+ (current-previous));
                System.err.println(String.format("Error processing dictionary:\n"
                        + "numBlocks = %d\n"
                        + "block = %d\n"
                        + "nextBlock = %d\n"
                        + "nextBytePos = %d\n"
                        + "bytePos = %d\n"
                        + "bufferPos = %d",
                        numBlocks,
                        block,
                        nextBlock,
                        nextBytePos,
                        bytePos,
                        nextBytePos - bytePos
                ));
                ex.printStackTrace();
                throw(ex);
            }

        } finally {
            try {
                out.close();
                file.delete();
            } catch (IOException e) {
                // TODO Auto-generated catch block
                e.printStackTrace();
            }
        }

    }

    /**
     * Locate the block of a string doing binary search.
     *
     * @param str
     * @return
     */
    protected int locateBlock(CharSequence str) {
        return 0;
    }

    /**
     *
     * @param str
     * @return
     */
    @Override
    public int locate(CharSequence str) {

        // Not found
        return 0;
    }

    /**
     *
     * @param blockid
     * @param str
     * @return
     */
    protected int locateInBlock(int blockid, CharSequence str) {

        return 0;
    }

    /**
     *
     * @param id
     * @return
     */
    @Override
    public CharSequence extract(int id) {

        return null;
    }

    /* (non-Javadoc)
	 * @see hdt.dictionary.DictionarySection#size()
     */
    @Override
    public long size() {
        return size;
    }

    /* (non-Javadoc)
	 * @see hdt.dictionary.DictionarySection#getNumberOfElements()
     */
    @Override
    public int getNumberOfElements() {
        return numstrings;
    }

    /* (non-Javadoc)
	 * @see hdt.dictionary.DictionarySection#getEntries()
     */
    @Override
    public Iterator<CharSequence> getSortedEntries() {
        return null;
    }

    /**
     *
     * @param output
     * @param listener
     * @throws IOException
     */
    @Override
    public void save(OutputStream output, ProgressListener listener) throws IOException {

        long dataLenght = 0;
        CRCOutputStream out = new CRCOutputStream(output, new CRC8());

        for(ArrayList<byte[]> block : this.data) {
            for (byte[] chunk : block) {
                dataLenght += chunk.length;
            }
        }

        out.write(TYPE_INDEX);
        VByte.encode(out, this.numstrings);
        VByte.encode(out, dataLenght);
        VByte.encode(out, this.blocksize);

        out.writeCRC();

        this.blocks.save(output, listener); // Write blocks directly to output, they have their own CRC check.

        out.setCRC(new CRC32());
        for(ArrayList<byte[]> block : this.data) {
            for (byte[] chunk : block) {
                dataLenght += chunk.length;
                IOUtil.writeBuffer(out, chunk, 0, chunk.length, listener);
            }
        }
        out.writeCRC();
    }

    protected void storeBuffer(int buffer) throws IOException {
        // System.out.println("Buffer = " + buffer);
        this.byteOut.flush();
        this.data.get(buffer).add(this.byteOut.toByteArray());
        this.byteOut.close();
    }

    @Override
    public void close() throws IOException {
        this.blocks.append(this.storedBuffersSize + this.byteOut.size());
        this.blocks.aggresiveTrimToSize();
        this.storeBuffer(this.buffer);
    }

    @Override
    public void load(TempDictionarySection other, ProgressListener listener) {
        throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
    }

    @Override
    public void load(InputStream input, ProgressListener listener) throws IOException {
        throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
    }
}
