package com.cbr.hdt.impl.spark;

import com.cbr.hdt.spark.Stats;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;
import org.rdfhdt.hdt.dictionary.DictionarySectionPrivate;
import org.rdfhdt.hdt.dictionary.impl.FourSectionDictionary;
import org.rdfhdt.hdt.hdt.HDTVocabulary;
import org.rdfhdt.hdt.header.Header;
import org.rdfhdt.hdt.listener.ProgressListener;
import org.rdfhdt.hdt.options.HDTOptions;

/**
 *
 * @author carlos
 */
public class SparkFourSectionDictionary extends FourSectionDictionary {

    public SparkFourSectionDictionary(HDTOptions spec,
            DictionarySectionPrivate s, DictionarySectionPrivate p, DictionarySectionPrivate o, DictionarySectionPrivate sh) {
        super(spec);
        this.subjects = s;
        this.predicates = p;
        this.objects = o;
        this.shared = sh;
    }

    public void loadFromSpark(final ProgressListener listener, final Map<String, Stats> stats) throws Exception {

        ExecutorService taskExecutor = Executors.newFixedThreadPool(4);
        List<Future> futures = new ArrayList<>();

        futures.add(taskExecutor.submit(new Callable<Integer>() {
            @Override
            public Integer call() throws Exception {
                ((SparkDictSection) subjects).loadFromSpark(null, stats);
                return 0;
            }
        }));
        futures.add(taskExecutor.submit(new Callable<Integer>() {
            @Override
            public Integer call() throws Exception {
                ((SparkDictSection) predicates).loadFromSpark(null, stats);
                return 0;
            }
        }));
        futures.add(taskExecutor.submit(new Callable<Integer>() {
            @Override
            public Integer call() throws Exception {
                ((SparkDictSection) objects).loadFromSpark(null, stats);
                return 0;
            }
        }));
        futures.add(taskExecutor.submit(new Callable<Integer>() {
            @Override
            public Integer call() throws Exception {
                ((SparkDictSection) shared).loadFromSpark(null, stats);
                return 0;
            }
        }));
        for (Future f : futures) {
            f.get();
        }
    }
    
    @Override
    public void populateHeader(Header header, String rootNode) {
		header.insert(rootNode, HDTVocabulary.DICTIONARY_TYPE, HDTVocabulary.DICTIONARY_TYPE_FOUR_SECTION);
		header.insert(rootNode, HDTVocabulary.DICTIONARY_NUMSUBJECTS, getNsubjects());
		header.insert(rootNode, HDTVocabulary.DICTIONARY_NUMPREDICATES, getNpredicates());
		header.insert(rootNode, HDTVocabulary.DICTIONARY_NUMOBJECTS, getNobjects());
		header.insert(rootNode, HDTVocabulary.DICTIONARY_NUMSHARED, getNshared());
//		header.insert(rootNode, HDTVocabulary.DICTIONARY_MAXSUBJECTID, getMaxSubjectID());
//		header.insert(rootNode, HDTVocabulary.DICTIONARY_MAXPREDICATEID, getMaxPredicateID());
//		header.insert(rootNode, HDTVocabulary.DICTIONARY_MAXOBJECTTID, getMaxObjectID());
		header.insert(rootNode, HDTVocabulary.DICTIONARY_SIZE_STRINGS, size());
	}

}
